% meta_UPDATE.m This script will update your metacognitive sensitivity 
% experiment. Your should call this script on each trial, after you record 
% a perceptual response (Perceptual_Resp) and a confidence rating 
% (Confidence_Rating) from your participant. Perceptual_Resp should be set 
% to a value of 1 or 2, whereas Confidence_Rating can be set to a value 
% from 0 to 1 (if you are using a continuous scale), or to any likert scale 
% rating (e.g. from 0 to 5). 

%% Store trial results
Exp_Results(Test_Number,1) = Condition_N;
Exp_Results(Test_Number,2) = Test_Inputs(Test_Level);
Exp_Results(Test_Number,3) = Perceptual_Resp-1;
Exp_Results(Test_Number,4) = Confidence_Rating;

%% Update staircase
Trial_Counts(Condition_N,Staircase_N)=Trial_Counts(Condition_N,Staircase_N)+1;

Staircase_Histories(Staircase_N,2:10,Condition_N)=Staircase_Histories(Staircase_N,1:9,Condition_N);
Staircase_Histories(Staircase_N,1,Condition_N)=Perceptual_Resp-1;

if Staircase_Histories(Staircase_N,1,Condition_N)~=Staircase_Histories(Staircase_N,2,Condition_N)
    % This staircase has reversed its response pattern
    Reversals(Condition_N,Staircase_N)=Reversals(Condition_N,Staircase_N)+1;
end

if Reversals(Condition_N,Staircase_N) == 0
    This_Step_Sz = 4*Min_Step_Size;
elseif Reversals(Condition_N,Staircase_N) == 1
    This_Step_Sz = 2*Min_Step_Size;
else
    This_Step_Sz = Min_Step_Size;
end

Median_Confidence = mean(Exp_Results(:,3));
Median_Weight = (Test_Number-1)/20;
if Median_Weight > 1
    Median_Weight=1;
end
Adj_Confidence_Crit = (Median_Confidence*Median_Weight) + (0.5*(Max_Possible_Confidence_Rating-Median_Weight));

if Staircase_N == 1 
    % Sample extremeties for lower category
    if Perceptual_Resp == 1 && sum(Staircase_Histories(Staircase_N,:,Condition_N)) < 1 
        Staircase_Sample_Pts(Condition_N,Staircase_N) = Staircase_Sample_Pts(Condition_N,Staircase_N)+This_Step_Sz;
    else
        Staircase_Sample_Pts(Condition_N,Staircase_N) = Staircase_Sample_Pts(Condition_N,Staircase_N)-This_Step_Sz;
    end
elseif Staircase_N == 2
    % Sample high-confidence boundary for lower category
    if Perceptual_Resp == 1 && Confidence_Rating > Adj_Confidence_Crit 
        Staircase_Sample_Pts(Condition_N,Staircase_N) = Staircase_Sample_Pts(Condition_N,Staircase_N)+This_Step_Sz;
    else
        Staircase_Sample_Pts(Condition_N,Staircase_N) = Staircase_Sample_Pts(Condition_N,Staircase_N)-This_Step_Sz;
    end
elseif Staircase_N == 3
    % Sample perceptual category boundary    
    if Perceptual_Resp == 1 
        Staircase_Sample_Pts(Condition_N,Staircase_N) = Staircase_Sample_Pts(Condition_N,Staircase_N)+This_Step_Sz;
    else
        Staircase_Sample_Pts(Condition_N,Staircase_N) = Staircase_Sample_Pts(Condition_N,Staircase_N)-This_Step_Sz;
    end
elseif Staircase_N == 4
    % Sample high-confidence boundary for upper category            
    if Perceptual_Resp == 2 & Confidence_Rating > Adj_Confidence_Crit 
        Staircase_Sample_Pts(Condition_N,Staircase_N) = Staircase_Sample_Pts(Condition_N,Staircase_N)-This_Step_Sz;
    else
        Staircase_Sample_Pts(Condition_N,Staircase_N) = Staircase_Sample_Pts(Condition_N,Staircase_N)+This_Step_Sz;
    end
else
    % Sample extremeties for upper category
    if Perceptual_Resp == 2 & sum(Staircase_Histories(Staircase_N,:,Condition_N)) > 9 
        Staircase_Sample_Pts(Condition_N,Staircase_N) = Staircase_Sample_Pts(Condition_N,Staircase_N)-This_Step_Sz;
    else
        Staircase_Sample_Pts(Condition_N,Staircase_N) = Staircase_Sample_Pts(Condition_N,Staircase_N)+This_Step_Sz;
    end
end

% Constrain staircases to test range
if Staircase_Sample_Pts(Condition_N,Staircase_N) < min(Test_Inputs) | Staircase_Sample_Pts(Condition_N,Staircase_N) < Test_Limit_Low
    Staircase_Sample_Pts(Condition_N,Staircase_N) = Staircase_Sample_Pts(Condition_N,Staircase_N)+This_Step_Sz;
elseif Staircase_Sample_Pts(Condition_N,Staircase_N) > max(Test_Inputs) | Staircase_Sample_Pts(Condition_N,Staircase_N) > Test_Limit_High
    Staircase_Sample_Pts(Condition_N,Staircase_N) = Staircase_Sample_Pts(Condition_N,Staircase_N)-This_Step_Sz;
end

%% Setup for the next trial
Test_Number = Test_Number+1;
Condition_N = ceil(Random_Trial_Order(Test_Number)/N_Trials_Per_Condition);
if Condition_N == 1
    Adj_Test_Number = 0;
else
    Adj_Test_Number = (Condition_N-1)*N_Trials_Per_Condition;
end

if Random_Trial_Order(Test_Number)-Adj_Test_Number < N_Trials_Per_Condition*0.2
    % Target lower performance asymptope - to ensure performance varies
    % from near 100% Lower categorisations...
    Staircase_N = 1;
elseif Random_Trial_Order(Test_Number)-Adj_Test_Number >= N_Trials_Per_Condition*0.2 && Random_Trial_Order(Test_Number)-Adj_Test_Number < N_Trials_Per_Condition*0.4
    % Target lower confidence boundary
    Staircase_N = 2;
elseif Random_Trial_Order(Test_Number)-Adj_Test_Number >= N_Trials_Per_Condition*0.4 && Random_Trial_Order(Test_Number)-Adj_Test_Number < N_Trials_Per_Condition*0.6
    % Target perceptual category boundary
    Staircase_N = 3;
elseif Random_Trial_Order(Test_Number)-Adj_Test_Number >= N_Trials_Per_Condition*0.6 && Random_Trial_Order(Test_Number)-Adj_Test_Number < N_Trials_Per_Condition*0.8
    % Target upper confidence boundary
    Staircase_N = 4;
else
    % Target upper performance asymptope - to ensure performance varies
    % upto near 100% Upper categorisations
    Staircase_N = 5;
end

[~,Test_Level] = find(Test_Inputs == Staircase_Sample_Pts(Condition_N,Staircase_N));
Test_Val = Staircase_Sample_Pts(Condition_N,Staircase_N);

